------------------
-- LUI class
local __lui       = {}
__lui.__index     = __lui
__lui.components  = {}
__lui.controls    = {}
__lui.config      = {}
__lui.delayBuffer = {}
__lui.addOnName   = 'LUI'
__lui.savedVars   = nil
__lui.savedVarsID = 17
__lui.LAM         = nil
__lui.LAMPanelID  = nil

function __lui:New( ... )
	local self = {}
	self = setmetatable( self, { __index = __lui } )
	self:Initialize()
	return self
end

function __lui:Initialize()
	EVENT_MANAGER:RegisterForEvent( self.addOnName, EVENT_ADD_ON_LOADED, function( eventCode, addOnName ) self:OnAddOnLoaded( addOnName ) end )
end

function __lui:InitControl( control )
	local name = control:GetName()
	self.components[ name ] = control
end

function __lui:OnAddOnLoaded( addOnName, addOnSubName )
	if ( addOnName ~= self.addOnName ) then return end
	local defaults = {
		InfoPanelEnabled  = true,
		MiniMapEnabled    = true,
		CombatInfoEnabled = true,
		UnitInfoEnabled   = true,
	}
	self.savedVars = ZO_SavedVars:NewAccountWide( string.format( '%s_%s', self.addOnName, 'SavedVariables' ), _LUI.savedVarsID, 'General', defaults )

	-- get LibAddonMenu
	if ( LibStub ~= nil ) then
		self.LAM = LibStub:GetLibrary( 'LibAddonMenu-1.0' )
		self.LAMPanelID = self.LAM:CreateControlPanel( self.addOnName, self.addOnName )
		self.LAM:AddHeader( self.LAMPanelID, string.format( '%s_GeneralHeader', self.addOnName ), 'General' )
	end

	-- init controls
	for name, control in pairs( self.components ) do
		if ( self.savedVars[ name..'Enabled' ] ) then
			self.controls[ name ] = _G[ '__'..name ]:New( control )
			control:SetHidden( false )

			-- add hide/show
			--local fragment = ZO_FadeSceneFragment:New( control )
			local fragment = ZO_SimpleSceneFragment:New( control )
			SCENE_MANAGER:GetScene( 'hud' ):AddFragment( fragment )
			SCENE_MANAGER:GetScene( 'hudui' ):AddFragment( fragment )
			SCENE_MANAGER:GetScene( 'siegeBar' ):AddFragment( fragment )
			-- SCENE_MANAGER.scenes.siegeBar:RegisterCallback( 'StateChange', function( oldState, newState ) 
				-- if( newState == 'showing' ) then
					-- control:SetHidden( false )
				-- end
			-- end )
		end
	end

	-- add general settings
	if ( self.LAM ~= nil ) then
		self.LAM:AddCheckbox( self.LAMPanelID, string.format( '%s_SettingsGeneralCheckbox_InfoPanel', self.addOnName ), 'Load InfoPanel', '',
			function() return self.savedVars[ 'InfoPanelEnabled' ] end,
			function() self.savedVars[ 'InfoPanelEnabled' ] = not self.savedVars[ 'InfoPanelEnabled' ] end,
		true, 'This needs an UI reload' )
		self.LAM:AddCheckbox( self.LAMPanelID, string.format( '%s_SettingsGeneralCheckbox_MiniMap', self.addOnName ), 'Load MiniMap', '',
			function() return self.savedVars[ 'MiniMapEnabled' ] end,
			function() self.savedVars[ 'MiniMapEnabled' ] = not self.savedVars[ 'MiniMapEnabled' ] end,
		true, 'This needs an UI reload' )
		self.LAM:AddCheckbox( self.LAMPanelID, string.format( '%s_SettingsGeneralCheckbox_CombatInfo', self.addOnName ), 'Load CombatInfo', '',
			function() return self.savedVars[ 'CombatInfoEnabled' ] end,
			function() self.savedVars[ 'CombatInfoEnabled' ] = not self.savedVars[ 'CombatInfoEnabled' ] end,
		true, 'This needs an UI reload' )
		self.LAM:AddCheckbox( self.LAMPanelID, string.format( '%s_SettingsGeneralCheckbox_UnitInfo', self.addOnName ), 'Load UnitInfo', '',
			function() return self.savedVars[ 'UnitInfoEnabled' ] end,
			function() self.savedVars[ 'UnitInfoEnabled' ] = not self.savedVars[ 'UnitInfoEnabled' ] end,
		true, 'This needs an UI reload' )
	end

	-- load component settings
	for name, control in pairs( self.controls ) do
		if ( self.LAM ~= nil ) then
			self.LAM:AddHeader( self.LAMPanelID, string.format( '%s_%sHeader', self.addOnName, name ), name )
			control:GetSettings( self.LAM, self.LAMPanelID, self.addOnName )
		end
	end
end

function __lui:Toggle( hidden )
	for name, control in pairs( self.controls ) do
		control.control:SetHidden( hidden )
	end
end

function __lui:DelayBuffer( key, buffer )
	if ( key == nil ) then return end
	if ( self.delayBuffer[key] == nil ) then self.delayBuffer[key] = {} end
	self.delayBuffer[key].buffer = buffer or 3
	self.delayBuffer[key].now = GetFrameTimeMilliseconds()
	if ( self.delayBuffer[key].last == nil ) then self.delayBuffer[key].last = self.delayBuffer[key].now end
	self.delayBuffer[key].eval = ( self.delayBuffer[key].now - self.delayBuffer[key].last ) >= self.delayBuffer[key].buffer
	if ( self.delayBuffer[key].eval ) then self.delayBuffer[key].last = self.delayBuffer[key].now end
	return self.delayBuffer[key].eval
end

function __lui:StringFormat( fmt, ... )
	local args, order = {...}, {}
	local fmt = fmt:gsub('%%(%d+)%$', function( i ) table.insert( order, args[ tonumber( i ) ] ) return '%' end)
	return string.format( fmt, unpack( order ) ) -- table.unpack for lua 5.2 / unpack for 5.1 or lower
end

function __lui:ParseLanguageString( value )
	return string.gsub( value, '%^(.+)', '' )
end

------------------
-- Initializer
_LUI = __lui:New()