------------------
-- CombatInfo class
__CombatInfo              = ZO_Object:Subclass()
__CombatInfo.savedVars    = nil
__CombatInfo.abilities    = setmetatable( {}, { __mode = 'kv' } )

-- -- exp/vp
__CombatInfo.lastExp      = 0
__CombatInfo.lastVp       = 0

-- -- events queue
__CombatInfo.queue         = {}
__CombatInfo.queue.enabled = {}
__CombatInfo.queue.events  = {}
__CombatInfo.queue.next    = {}

-- -- random position grid
__CombatInfo.randomPos    = {}

function __CombatInfo:New( ... )
	local result = ZO_Object.New( self )
	result:Initialize( ... )
	return result
end

function __CombatInfo:Initialize( control )
	-- set controls
	self.control = control

	-- load settings
	local defaults = {
		HUDElements = true,
		CoolDown    = true,
	}
	self.savedVars = ZO_SavedVars:NewAccountWide( string.format( '%s_%s', _LUI.addOnName, 'SavedVariables' ), _LUI.savedVarsID, 'CombatInfo', defaults )
	CombatInfo_HUDElements:SetHidden( not self.savedVars.HUDElements )

	-- set control defaults
	CombatInfo_Labels_PlayerDivider:SetAddressMode( TEX_MODE_CLAMP )
	CombatInfo_Labels_PlayerDivider:SetBlendMode( TEX_BLEND_MODE_ADD )
	CombatInfo_Labels_PlayerDivider:SetColor( _LUI.config.colors.WHITE.r, _LUI.config.colors.WHITE.g, _LUI.config.colors.WHITE.b, CombatInfo_Labels_PlayerDivider:GetAlpha() )
	CombatInfo_Labels_OtherDivider:SetAddressMode( TEX_MODE_CLAMP )
	CombatInfo_Labels_OtherDivider:SetBlendMode( TEX_BLEND_MODE_ADD )
	CombatInfo_Labels_OtherDivider:SetColor( _LUI.config.colors.WHITE.r, _LUI.config.colors.WHITE.g, _LUI.config.colors.WHITE.b, CombatInfo_Labels_OtherDivider:GetAlpha() )

	-- create event queue tables
	-- todo: get targets from config.lua and iterate over them
	self.queue.events[ 'Target' ]       = {}
	self.queue.events[ 'TargetInfo' ]   = {}
	self.queue.events[ 'PlayerDown' ]   = {}
	self.queue.events[ 'PlayerUp' ]     = {}
	self.queue.events[ 'OtherDown' ]    = {}
	self.queue.events[ 'OtherUp' ]      = {}
	self.queue.events[ 'Magicka' ]      = {}
	self.queue.events[ 'Health' ]       = {}
	self.queue.events[ 'Stamina' ]      = {}
	self.queue.events[ 'MountStamina' ] = {}
	self.queue.events[ 'Ultimate' ]     = {}

	-- set next queue fire defaults
	-- todo: get targets from config.lua and iterate over them
	self.queue.next[ 'Target' ]       = true
	self.queue.next[ 'TargetInfo' ]   = true
	self.queue.next[ 'PlayerDown' ]   = true
	self.queue.next[ 'PlayerUp' ]     = true
	self.queue.next[ 'OtherDown' ]    = true
	self.queue.next[ 'OtherUp' ]      = true
	self.queue.next[ 'Magicka' ]      = true
	self.queue.next[ 'Health' ]       = true
	self.queue.next[ 'Stamina' ]      = true
	self.queue.next[ 'MountStamina' ] = true
	self.queue.next[ 'Ultimate' ]     = true

	-- set randomPosition grid defaults
	self.randomPos[ 'Target' ] = {}
	self.randomPos[ 'Target' ].last = 0
	self.randomPos[ 'Target' ].max  = 9
	self.randomPos[ 'Target' ].size = 70
	self.randomPos[ 'Target' ][ 1 ] = {}
	self.randomPos[ 'Target' ][ 1 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 1 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 2 ] = {}
	self.randomPos[ 'Target' ][ 2 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 2 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 3 ] = {}
	self.randomPos[ 'Target' ][ 3 ].offsetX = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'Target' ][ 3 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 4 ] = {}
	self.randomPos[ 'Target' ][ 4 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 4 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 5 ] = {}
	self.randomPos[ 'Target' ][ 5 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 5 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 6 ] = {}
	self.randomPos[ 'Target' ][ 6 ].offsetX = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'Target' ][ 6 ].offsetY = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 7 ] = {}
	self.randomPos[ 'Target' ][ 7 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 )
	self.randomPos[ 'Target' ][ 7 ].offsetY = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'Target' ][ 8 ] = {}
	self.randomPos[ 'Target' ][ 8 ].offsetX = 0 - ( self.randomPos[ 'Target' ].size / 2 + self.randomPos[ 'Target' ].size )
	self.randomPos[ 'Target' ][ 8 ].offsetY = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'Target' ][ 9 ] = {}
	self.randomPos[ 'Target' ][ 9 ].offsetX = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'Target' ][ 9 ].offsetY = self.randomPos[ 'Target' ].size / 2
	self.randomPos[ 'TargetInfo' ] = {}
	self.randomPos[ 'TargetInfo' ].last = 0
	self.randomPos[ 'TargetInfo' ].max  = 3
	self.randomPos[ 'TargetInfo' ].size  = 100
	self.randomPos[ 'TargetInfo' ][ 1 ] = {}
	self.randomPos[ 'TargetInfo' ][ 1 ].offsetX = 0 - ( self.randomPos[ 'TargetInfo' ].size / 2 )
	self.randomPos[ 'TargetInfo' ][ 1 ].offsetY = 0 - ( self.randomPos[ 'TargetInfo' ].size / 2 )
	self.randomPos[ 'TargetInfo' ][ 2 ] = {}
	self.randomPos[ 'TargetInfo' ][ 2 ].offsetX = 0 - ( self.randomPos[ 'TargetInfo' ].size / 2 + self.randomPos[ 'TargetInfo' ].size )
	self.randomPos[ 'TargetInfo' ][ 2 ].offsetY = 0 - ( self.randomPos[ 'TargetInfo' ].size / 2 )
	self.randomPos[ 'TargetInfo' ][ 3 ] = {}
	self.randomPos[ 'TargetInfo' ][ 3 ].offsetX = self.randomPos[ 'TargetInfo' ].size / 2
	self.randomPos[ 'TargetInfo' ][ 3 ].offsetY = 0 - ( self.randomPos[ 'TargetInfo' ].size / 2 )

	-- get ability data
	for i=1, GetNumAbilities() do
		local name, icon = GetAbilityInfoByIndex( i )
		self.abilities[ name ] = icon
	end

	-- get exp data
	self.lastExp = GetUnitXP( 'player' )
	self.lastVp  = GetUnitVeteranPoints( 'player' )

	-- set ability cooldown controls
	for actionButtonNum = 3, 8 do
		local parent = WINDOW_MANAGER:GetControlByName( string.format( 'ActionButton%d', actionButtonNum ) )
		local bg     = WINDOW_MANAGER:CreateControl( string.format( 'CombatInfo_ActionButton%d', actionButtonNum ), parent, CT_STATUSBAR )
		local label  = bg:CreateControl( string.format( 'CombatInfo_ActionButton%d_Label', actionButtonNum ), CT_LABEL )
		local size   = parent:GetWidth() - 4
		bg:SetHidden( true )
		bg:SetDimensions( size, size )
		bg:SetAnchor( CENTER, parent, CENTER, 0, 0 )
		bg:SetAlpha( 0.55 )
		bg:SetDrawLayer( 2 )
		label:SetHidden( true )
		label:SetFont( 'LUIFontAnnounceMedium' )
		label:SetText( '0.0' )
		label:SetAnchor( CENTER )
		label:SetDrawLayer( 2 )
	end

	-- set event handlers
	self.control:RegisterForEvent( EVENT_COMBAT_EVENT ,          function( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log ) self:OnCombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log ) end )
	self.control:RegisterForEvent( EVENT_EXPERIENCE_UPDATE,      function( reason, unitTag, currentExp, maxExp ) self:OnExperienceUpdate( 'exp', reason, unitTag, currentExp, maxExp ) end )
	self.control:RegisterForEvent( EVENT_VETERAN_POINTS_UPDATE,  function( unitTag, currentPoints, maxPoints, reason ) self:OnExperienceUpdate( 'vp', unitTag, currentPoints, maxPoints, reason ) end )
	self.control:SetHandler( 'OnUpdate', function() self:OnUpdate() end )
end

function __CombatInfo:OnUpdate()
	-- fire events from combat-queue
	for target, v in pairs( self.queue.events ) do
		if ( self.queue.events[ target ][ 1 ] ~= nil and self.queue.next[ target ] ) then
			local event = self.queue.events[ target ][ 1 ]
			self:FireCombatEvent( event.eventCode, event.result, event.isError, event.abilityName, event.abilityGraphic, event.abilityActionSlotType, event.sourceName, event.sourceType, event.targetName, event.targetType, event.hitValue, event.powerType, event.damageType, event.log )
			self.queue.next[ target ] = false
			table.remove( self.queue.events[ target ], 1 )
		end
	end

	-- limit update speed
	if not _LUI:DelayBuffer( 'CombatInfoUpdate', 40 ) then return end

	-- ability cooldowns
	if ( self.savedVars.CoolDown ) then
		for actionButtonNum = 3, 8 do
			local bg    = WINDOW_MANAGER:GetControlByName( string.format( 'CombatInfo_ActionButton%d', actionButtonNum ) )
			local label = WINDOW_MANAGER:GetControlByName( string.format( 'CombatInfo_ActionButton%d_Label', actionButtonNum ) )
			local remain, duration, global = GetSlotCooldownInfo( actionButtonNum )
			if ( duration > 0 ) then
				remain = remain / 1000
				bg:SetHidden( false )
				bg:SetAlpha( remain )
				label:SetHidden( false )
				label:SetText( string.format( '%.1f', remain ) )
			else
				bg:SetHidden( true )
			end
		end
	end

	-- loop label targets
	local currentTime = GetGameTimeMilliseconds()
	local isAlive = false
	local numTargetChildren = CombatInfo_Labels:GetNumChildren()
	for targetnum = 1, numTargetChildren do
		-- get label target
		local target = CombatInfo_Labels:GetChild( targetnum )

		-- loop labels
		local numLabelChildren = target:GetNumChildren()
		local queueNextCount   = 0
		for labelnum = 1, numLabelChildren do
			-- get label
			local label = target:GetChild( labelnum )

			-- -- handle next queue fire
			if ( label.settings.nextIn ~= nil ) then
				if ( label.settings.nextIn <= 0 ) then
					label.settings.nextIn = nil
					self.queue.next[ label.settings.target ] = true

				-- keep removing from labels nextIn value, even if label is already invisible
				elseif ( label.settings.nextIn > 0 ) then
					-- todo: this is bugged, needs to keep an eye on fixedAge ...
					label.settings.nextIn = label.settings.nextIn - label.settings.animateSpeed
				end
			end

			-- animate label
			if ( label.settings.age ~= 0 ) then
				if ( label.settings.age <= currentTime ) then
					-- remove label
					label:ClearAnchors()
					label:SetHidden( true )
					label.settings.age = 0
				else
					-- animation phase
					if ( label.settings.fixedAge <= currentTime ) then
						-- alpha fade
						if ( label.settings.animateAlpha == true ) then
							local alpha = ( ( label.settings.age - currentTime ) / label.settings.animateTime )
							alpha = 1.0 * alpha * alpha
							label:SetAlpha( alpha )
						end

						-- movement
						if ( label.settings.animateX > 0 or label.settings.animateY > 0 ) then
							if ( label.settings.animateX == 1 ) then     -- left
								label.settings.animateXoffset = label.settings.animateXoffset - label.settings.animateSpeed
							elseif ( label.settings.animateX == 2 ) then -- right
								label.settings.animateXoffset = label.settings.animateXoffset + label.settings.animateSpeed
							end
							if ( label.settings.animateY == 1 ) then     -- up
								label.settings.animateYoffset = label.settings.animateYoffset - label.settings.animateSpeed
							elseif ( label.settings.animateY == 2 ) then -- down
								label.settings.animateYoffset = label.settings.animateYoffset + label.settings.animateSpeed
							end
							label:ClearAnchors()
							label:SetAnchor( label.settings.align, label.settings.targetControl, label.settings.align, label.settings.animateXoffset, label.settings.animateYoffset )
						end
					-- fixed phase
					else
						-- scale fade
						if ( label.settings.animateScale == true ) then
							local scale = ( ( label.settings.fixedAge - currentTime ) / label.settings.fixedTime )
							scale = 1.0 * scale * scale
							label:SetScale( 1.0 + scale )
						end
					end

					-- keep HUD alive
					isAlive = true
				end
			end
		end
	end

	-- keep HUD alive or hide it
	if ( self.savedVars.HUDElements ) then
		CombatInfo_HUDElements:SetHidden( not isAlive )
	end
end

function __CombatInfo:OnExperienceUpdate( mode, reason, unitTag, currentPoints, maxPoints )
	if ( unitTag ~= 'player' ) then return end

	-- calc exp
	local value = 0
	local eventID = 99999999997
	if ( mode == 'vp' ) then
		eventID      = 99999999998
		value        = currentPoints - self.lastExp
		self.lastExp = currentPoints
	elseif ( mode == 'exp' ) then
		eventID      = 99999999997
		value        = currentPoints - self.lastVp
		self.lastVp  = currentPoints
	end

	-- send custom combat event
	if ( value < 1 ) then return end
	self:OnCombatEvent( 0, eventID, false, mode, '', 0, '', COMBAT_UNIT_TYPE_PLAYER, '', COMBAT_UNIT_TYPE_PLAYER, value, 0, 0, false )
end

function __CombatInfo:OnCombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )
	-- filter events by eventId, sourceType, targetType
	--CONSOLE:LogInfo( string.format( 's%d/t%d (%s>%s) / res: %d / hit: %d', sourceType, targetType, sourceName, targetName, result, hitValue) )
	if ( not self:FilterEvent( result, sourceType, targetType, powerType ) ) then return end

	-- get event config
	local target = nil
	if ( _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].powers ~= nil ) then
		target = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].target[ powerType ]
	else
		target = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].target
	end

	-- get target config
	local targetConfig = _LUI.config.combatTargets[ target ]

	-- add to queue or fire instant
	if ( targetConfig.queue ) then
		-- create event table
		local combatEvent          = {}
		combatEvent.eventCode      = eventCode
		combatEvent.result         = result
		combatEvent.isError        = isError
		combatEvent.abilityName    = abilityName
		combatEvent.abilityGraphic = abilityGraphic
		combatEvent.abilityAST     = abilityActionSlotType
		combatEvent.sourceName     = sourceName
		combatEvent.sourceType     = sourceType
		combatEvent.targetName     = targetName
		combatEvent.targetType     = targetType
		combatEvent.hitValue       = hitValue
		combatEvent.powerType      = powerType
		combatEvent.damageType     = damageType
		combatEvent.log            = log
		combatEvent.target         = target

		-- add event to queue
		table.insert( self.queue.events[ target ], combatEvent )
	else
		self:FireCombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )
	end
end

function __CombatInfo:FireCombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )
	-- parse names
	sourceName = _LUI:ParseLanguageString( sourceName )
	targetName = _LUI:ParseLanguageString( targetName )

	-- get event config
	local target = nil
	local color  = nil
	local font   = nil
	local strfmt = nil
	if ( _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].powers ~= nil ) then
		target = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].target[ powerType ]
		color  = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].color[ powerType ]
		font   = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].font[ powerType ]
		strfmt = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].strfmt[ powerType ]
	else
		target = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].target
		color  = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].color
		font   = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].font
		strfmt = _LUI.config.combatEvents[ result ][ sourceType ][ targetType ].strfmt
	end

	-- get target config
	local targetConfig  = _LUI.config.combatTargets[ target ]
	local targetControl = CombatInfo_Labels:GetNamedChild( string.format( '_%s', target ) )

	-- get label
	local label         = nil
	local icon          = nil
	targetConfig.labelsCount = ( targetConfig.labelsCount == targetConfig.labelsMax ) and 1 or targetConfig.labelsCount + 1
	if ( targetControl:GetNamedChild( string.format( '_%s', targetConfig.labelsCount ) ) == nil ) then
		label = WINDOW_MANAGER:CreateControl( string.format( 'CombatInfo_Labels_%s_%s', target, targetConfig.labelsCount ), targetControl, CT_LABEL )
		icon  = label:CreateControl( string.format( 'CombatInfo_Labels_%s_i%s', target, targetConfig.labelsCount ), CT_TEXTURE )
	else
		label = targetControl:GetNamedChild( string.format( '_%s', targetConfig.labelsCount ) )
		icon  = label:GetChild( 1 )
	end

	-- get previous label
	local previousNum   = ( targetConfig.labelsCount - 1 < 1 ) and targetConfig.labelsMax or targetConfig.labelsCount - 1
	local previousLabel = targetControl:GetChild( previousNum )

	-- label settings
	label.settings					= {}
	label.settings.target			= target
	label.settings.targetControl	= targetControl
	label.settings.showIcon			= targetConfig.showIcon
	label.settings.animateAlpha		= targetConfig.animateAlpha
	label.settings.animateScale		= targetConfig.animateScale
	label.settings.animateY			= targetConfig.animateY
	label.settings.animateX			= targetConfig.animateX
	label.settings.animateSpeed		= targetConfig.animateSpeed
	label.settings.animateTime		= targetConfig.animateTime
	label.settings.fixedTime		= targetConfig.fixedTime
	label.settings.align			= targetConfig.align
	label.settings.alignIcon		= targetConfig.alignIcon
	label.settings.alignIconTarget	= targetConfig.alignIconTarget
	label.settings.alignIconOffsetX	= targetConfig.alignIconOffsetX
	label.settings.alignIconOffsetY	= targetConfig.alignIconOffsetY
	if( targetConfig.grid ) then
		label.settings.animateXoffset,
		label.settings.animateYoffset = self:GetRandomPositionByGrid( target )
	else
		label.settings.animateXoffset = 0
		label.settings.animateYoffset = 0
	end
	label.settings.nextIn			= ( previousLabel ~= nil ) and previousLabel:GetHeight() or 0
	label.settings.fixedAge			= GetGameTimeMilliseconds() + label.settings.fixedTime
	label.settings.age				= label.settings.fixedAge + label.settings.animateTime

	-- label setup
	label:SetAlpha( 0 )
	label:SetDrawLayer( DL_TEXT )
	label:SetColor( color.r, color.g, color.b )
	label:SetFont( font )
	label:SetText( _LUI:StringFormat( strfmt, hitValue, sourceName, targetName ) )
	label:ClearAnchors()
	label:SetAnchor( label.settings.align, label.settings.targetControl, label.settings.align, label.settings.animateXoffset, label.settings.animateYoffset )
	label:SetHidden( false )

	-- get ability texture
	local iconTexture = ''
	if ( abilityName ~= '' and self.abilities[ abilityName ] ~= nil ) then
		iconTexture = self.abilities[ abilityName ]
	end

	-- icon setup
	local sizeY = label:GetTextHeight()
	icon:SetDimensions( sizeY, sizeY )
	icon:SetAnchor( label.settings.alignIcon, label, label.settings.alignIconTarget, label.settings.alignIconOffsetX, label.settings.alignIconOffsetY )
	icon:SetTexture( iconTexture )
	icon:SetHidden( ( iconTexture == '' or label.settings.showIcon == false ) and true or false )
end

function __CombatInfo:FilterEvent( result, sourceType, targetType, powerType )
	-- only events from the combatEvents table with defined sourceType/targetType combination
	for k,v in pairs( _LUI.config.combatEvents ) do
		if ( result == k ) then
			for k2,v2 in pairs( _LUI.config.combatEvents[k] ) do
				if ( sourceType == k2 ) then
					for k3,v3 in pairs( _LUI.config.combatEvents[k][k2] ) do
						if ( targetType == k3 ) then
							if ( _LUI.config.combatEvents[ k ][ k2 ][ k3 ].powers ~= nil ) then
								for k4,v4 in pairs( _LUI.config.combatEvents[k][k2][k3].target ) do
									if ( powerType == k4 ) then
										return true
									end
								end
							else
								return true
							end
						end
					end
				end
			end
		end
	end
	return false
end

function __CombatInfo:GetRandomPositionByGrid( target )
	local gridIndex = ( self.randomPos[ target ].last >= self.randomPos[ target ].max ) and 1 or self.randomPos[ target ].last + 1
	self.randomPos[ target ].last = gridIndex
	local offsetX   = self.randomPos[ target ][ gridIndex ].offsetX
	local offsetY   = self.randomPos[ target ][ gridIndex ].offsetY
	offsetX         = 0 - math.random( offsetX, offsetX + self.randomPos[ target ].size )
	offsetY         = 0 - math.random( offsetY, offsetY + self.randomPos[ target ].size )
	return offsetX, offsetY
end


function __CombatInfo:GetSettings( LAM, LAMPanelID, addOnName )
	LAM:AddCheckbox( LAMPanelID, string.format( '%s_SettingsCombatInfoHudLinesCheckbox', addOnName ), 'Show additional HUD elements', '',
		function() return self.savedVars.HUDElements end,
		function()
			self.savedVars.HUDElements = not self.savedVars.HUDElements
			CombatInfo_HUDElements:SetHidden( not self.savedVars.HUDElements )
		end,
	false, '' )
	LAM:AddCheckbox( LAMPanelID, string.format( '%s_SettingsCombatInfoCoolDownCheckbox', addOnName ), 'Show CoolDown Information', '',
		function() return self.savedVars.CoolDown end,
		function()
			self.savedVars.CoolDown = not self.savedVars.CoolDown
		end,
	false, '' )
end