------------------
-- InfoPanel class
__InfoPanel             = ZO_Object:Subclass()
__InfoPanel.savedVars   = nil
__InfoPanel.fps_ranges  = {
	[ 1 ] = { range = { min = 0,  max = 24   }, color = _LUI.config.colors.RED    },
	[ 2 ] = { range = { min = 25, max = 55   }, color = _LUI.config.colors.YELLOW },
	[ 3 ] = { range = { min = 56, max = 9999 }, color = _LUI.config.colors.GREEN  },
}
__InfoPanel.dura_ranges = {
	[ 1 ] = { range = { min = 0,  max = 49  }, color = _LUI.config.colors.RED    },
	[ 2 ] = { range = { min = 50, max = 79  }, color = _LUI.config.colors.YELLOW },
	[ 3 ] = { range = { min = 80, max = 100 }, color = _LUI.config.colors.GREEN  },
}
__InfoPanel.bags_ranges = {
	[ 1 ] = { range = { min = 0,  max = 49  }, color = _LUI.config.colors.WHITE  },
	[ 2 ] = { range = { min = 50, max = 89  }, color = _LUI.config.colors.YELLOW },
	[ 3 ] = { range = { min = 90, max = 100 }, color = _LUI.config.colors.RED    },
}

function __InfoPanel:New( ... )
	local result = ZO_Object.New( self )
	result:Initialize( ... )
	return result
end

function __InfoPanel:Initialize( control )
	self.control = control

	-- load settings
	local defaults = {
		offsetX = self.control:GetRight() - GuiRoot:GetRight(),
		offsetY = self.control:GetTop(),
		scale   = 1,
	}
	self.savedVars = ZO_SavedVars:NewAccountWide( string.format( '%s_%s', _LUI.addOnName, 'SavedVariables' ), _LUI.savedVarsID, 'InfoPanel', defaults )
	self.control:SetScale( self.savedVars.scale )
	self.control:ClearAnchors()
	self.control:SetAnchor( TOPRIGHT, GuiRoot, TOPRIGHT, self.savedVars.offsetX, self.savedVars.offsetY )
	self.control:SetHandler( 'OnMoveStop', function()
		self.savedVars.offsetX = self.control:GetRight() - GuiRoot:GetRight()
		self.savedVars.offsetY = self.control:GetTop()
	end )

	-- set init values
	InfoPanel_World:SetText( GetWorldName() )
	self:OnUpdate01()
	self:OnUpdate60()

	-- set event handlers
	self.control:RegisterForEvent( EVENT_LOOT_RECEIVED, function( ) self:OnBagUpdate() end )
	self.control:RegisterForEvent( EVENT_INVENTORY_SINGLE_SLOT_UPDATE, function( ) self:OnBagUpdate() end )
	self.control:RegisterForEvent( EVENT_STABLE_INTERACT_END, function( ) self:OnUpdate60() end )
	self.control:SetHandler( 'OnUpdate', function() self:OnUpdate() end )
end

function __InfoPanel:OnUpdate()
	-- update once a second
	if _LUI:DelayBuffer( 'InfoPanelOnUpdate01', 1000 ) then
		self:OnUpdate01()
	end

	-- update once a minute
	if _LUI:DelayBuffer( 'InfoPanelOnUpdate60', 60000 ) then
		self:OnUpdate60()
	end
end

function __InfoPanel:OnBagUpdate()
	-- update bags
	local totalSlots = 0
	local totalItems = 0
	local numBags    = GetMaxBags()
	for bagNum = 1, numBags do
		local icon, numSlots = GetBagInfo( bagNum )
		for slotNum = 0, numSlots do
			local count = GetItemTotalCount( bagNum, slotNum )
			totalItems = ( count > 0 ) and totalItems + 1 or totalItems
		end
		totalSlots = totalSlots + numSlots
	end

	local filledSlotPercentage = ( totalItems / totalSlots ) * 100
	for k, v in pairs( self.bags_ranges ) do
		if ( filledSlotPercentage >= v.range.min and filledSlotPercentage <= v.range.max ) then
			InfoPanel_LabelBags_Label:SetColor( v.color.r, v.color.g, v.color.b, 1 )
			break
		end
	end
	InfoPanel_LabelBags_Label:SetText( string.format( '%d/%d', totalItems, totalSlots ) )
end

function __InfoPanel:OnUpdate01()
	-- update time
	InfoPanel_Time:SetText( GetTimeString() )

	-- update fps
	local fps = GetFramerate()
	for k, v in pairs( self.fps_ranges ) do
		if ( fps >= v.range.min and fps <= v.range.max ) then
			InfoPanel_FPS:SetColor( v.color.r, v.color.g, v.color.b, 1 )
			break
		end
	end
	InfoPanel_FPS:SetText( string.format( '%.1f fps', fps ) )
end

function __InfoPanel:OnUpdate60()
	-- update mountfeedtimer
	local mountFeedTimer   = GetTimeUntilMountCanBeFed()
	local mountFeedMessage = 'No mount'
	if ( mountFeedTimer ~= nil ) then 
		local timer   = mountFeedTimer / 1000
		local hours   = math.floor( mountFeedTimer / 3600000 )
		local minutes = math.floor( ( timer - ( hours * 3600 ) ) / 60 )
		if ( mountFeedTimer == 0 ) then
			mountFeedMessage = 'Feed now'
		elseif ( mountFeedTimer > 0 ) then
			mountFeedMessage = string.format( '%dh %dm', hours, minutes )
		end
	end
	InfoPanel_MountFeed_Label:SetText( mountFeedMessage )

	-- update item durability
	local slotCount    = 0
	local duraSum      = 0
	local missingSlots = 0
	for k,v in pairs( _LUI.config.equipmentSlots ) do
		if ( DoesItemHaveDurability( 0, v ) == true ) then
			duraSum = duraSum + GetItemCondition( 0, v )
			slotCount = slotCount + 1
		end
	end
	local duraPercentage = ( slotCount == 0 ) and 0 or duraSum / slotCount
	for k, v in pairs( self.dura_ranges ) do
		if ( duraPercentage >= v.range.min and duraPercentage <= v.range.max ) then
			InfoPanel_LabelDurability_Label:SetColor( v.color.r, v.color.g, v.color.b, 1 )
			break
		end
	end
	InfoPanel_LabelDurability_Label:SetText( string.format( '%d%%', duraPercentage ) )

	-- update bag slot count
	self:OnBagUpdate()
end

function __InfoPanel:GetSettings( LAM, LAMPanelID, addOnName )
	LAM:AddSlider( LAMPanelID, string.format( '%s_SettingsInfoPanelScaleSlider', addOnName ), 'Scale %', '', 50, 200, 10,
		function() return self.savedVars.scale * 100  end,
		function( value ) self.savedVars.scale = value / 100; self.control:SetScale( self.savedVars.scale ) end,
	false, '' )
end